/*
** ############################################################################
**
**		Filename  : DEMOAC256_QS.c
**		Compiler  : Freescale CodeWarrior C Compiler
**		Date      : March 2008
**
**#############################################################################     
**
**    Quick Start Demo code for the MCF51AC256 MCU installed on DEMOAC board 
**		from Axiom Manufacturing.  This code may be freely distributed or modified.   
**		No functionality or usability is implied or guaranteed.
**
**		Visit us at      http://www.axman.com
**
** ############################################################################
*/

#include <hidef.h>                        /* for EnableInterrupts macro */
#include "derivative.h"                   /* include peripheral declarations */
#include "DEMOAC256_QS.h"                 /* include variable declarations */

void InitSystems() {
  
  SOPT   = 0x20;                          // Disable COP
  SOPT2  = 0x00;                          // TPM clock supplied to FTMx
  SPMSC1 = 0x00;                          // Disable LVD
  SPMSC2 = 0x00;                          // Disable power-down modes
  SCGC1  = 0xFF;                          // Enable bus clock to peripherals
  SCGC2  = 0xFF;                          // Enable bus clock to peripherals
}                                         // end InitSystems

void EnablePullups() {
  
  PTAPE = 0xFF;                           // Enable PORT A Internal Pullups
  PTBPE = 0xFF;                           // Enable PORT B Internal Pullups
  PTCPE = 0xFF;                           // Enable PORT C Internal Pullups
  PTDPE = 0xFF;                           // Enable PORT D Internal Pullups
  PTEPE = 0xFF;                           // Enable PORT E Internal Pullups
  PTFPE = 0xFF;                           // Enable PORT F Internal Pullups
  PTGPE = 0xFF;                           // Enable PORT G Internal Pullups
  PTHPE = 0xFF;                           // Enable PORT H Internal Pullups
  PTJPE = 0xFF;                           // Enable PORT J Internal Pullups
}                                         // end EnablePullups

void InitKBI() {
  
  /*KBI Init*/
  KBISC_KBIE   = 0;                        // Mask KBI interrupts
                                           
                                              
  KBIPE_KBIPE3 = 1;                        // Enable KBI1P3 interrupts  
  KBIPE_KBIPE2 = 1;                        // Enable KBI1P2 interrupts  
  KBIPE_KBIPE1 = 1;                        // Enable KBI1P1 interrupts  
  KBIPE_KBIPE0 = 1;                        // Enable KBI1P0 interrupts  
  KBISC_KBIMOD = 0;                        // Detect edges only
  KBISC_KBACK  = 1;                        // Clear pending interrupts
  KBISC_KBIE   = 1;                        // Enable KBI1 interrupts
  
}                                          // end InitKBI

void InitADC(void) {
  
  byte done = 0x00;
  ADC1SC1 = 0x1F;                          /* Disable interrupts 
                                              Disable continuous conversions
                                              Disable ADC by setting all ADCH bits */
  ADC1SC2 = 0x40;                          /* Select H/W trigger
                                              Disable compare function */
  ADC1CFG = 0x03; 	                      /* ADIV=input clock/1
                                              ADLSMP=short sample time
                                              MODE=8-bit conversion
                                              ADICLK=async clock */
  APCTL2 = 0x01;                           // Enable ADP8 as ADC input
}                                          // end InitADC

void SetADC(byte adc_channel, byte aien_value) {

  ADC1SC1_AIEN = aien_value&0x01;			 // enable ADC interrupt 
  ADC1SC1_ADCH = adc_channel;					 // select ADC channel
}                                          // end SetADC

// Initialize the FTM1 for PWM outputs (Channel 1 for Buzzer)
void FTM1Cx_PWM(byte CHAN, byte DUTY_CYCLE, word MOD, byte CLK_PS) {

    int val;
    
    if (MOD==0) {
       FTM1SC = 0x00;                      // disable PWM.
       return; 
    }
    if ((FTM1MODH==(MOD>>8)) && (FTM1MODL==(MOD-1))) {
    } else {
       FTM1MODH = (byte) (MOD>>8);			 // set period 
       FTM1MODL = (byte) (MOD-1);
    }
    
    switch (CHAN) {

      case 0: 
           FTM1C0SC = 0x24;                // Center-aligned low-true pulses
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM1C0VH = (byte) (val>>8);     // set duty cycle for Channel 0
           FTM1C0VL = (byte) (val);
      break;
        
      case 1:
           FTM1C1SC = 0x24;                // Center-aligned low-true pulses
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM1C1VH = (byte) (val>>8);     // set duty cycle for Channel 1
           FTM1C1VL = (byte) (val);			 
      break;
        
      case 2:
           FTM1C2SC = 0x24;                // Center-aligned low-true pulses
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM1C2VH = (byte) (val>>8);     // set duty cycle for Channel 2
           FTM1C2VL = (byte) (val);
      break;

      case 3: 
           FTM1C3SC = 0x24;                // Center-aligned low-true pulses
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM1C3VH = (byte) (val>>8);     // set duty cycle for Channel 0
           FTM1C3VL = (byte) (val);
      break;
        
      case 4:
           FTM1C4SC = 0x24;                // Center-aligned low-true pulses
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM1C4VH = (byte) (val>>8);     // set duty cycle for Channel 1
           FTM1C4VL = (byte) (val);
      break;
        
      case 5:
           FTM1C5SC = 0x24;                // Center-aligned low-true pulses
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM1C5VH = (byte) (val>>8);     // set duty cycle for Channel 2
           FTM1C5VL = (byte) (val);
      break;
        
      default:
      break;      
    }
    
    FTM1SC   = (byte) (0x08|(CLK_PS&0x07));  // individual aligned, bus frequency/Prescale, start timer
} //end FTM1Cx_PWM

// Initialize the FTM2 Channels 0-5 for PWM outputs (LEDs)
void FTM2Cx_PWM(byte CHAN, byte DUTY_CYCLE, word MOD, byte CLK_PS) {

    int val;
    
    if (MOD==0) {
      
       FTM2SC=0x00;                        // disable PWM.
       return; 
    }
    if ((FTM2MODH==(MOD>>8)) && (FTM2MODL==(MOD-1))) {
      
    } else {
      
       FTM2MODH = (byte) (MOD>>8);		    // set period 
       FTM2MODL = (byte) (MOD-1);
    }
    
    
    switch (CHAN) {
    
      case 0: 
           FTM2C0SC = 0x24;                // Center-aligned low-true pulses
           
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM2C0VH = (byte) (val>>8);     // set duty cycle for Channel 0
           FTM2C0VL = (byte) (val);
      break;
        
      case 1:
           FTM2C1SC = 0x24;                // Center-aligned low-true pulses
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM2C1VH = (byte) (val>>8);     // set duty cycle for Channel 1
           FTM2C1VL = (byte) (val);
      break;
        
      case 2:
           FTM2C2SC = 0x24;                // Center-aligned low-true pulses
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM2C2VH = (byte) (val>>8);     // set duty cycle for Channel 2
           FTM2C2VL = (byte) (val);
      break;
        
      case 3:
           FTM2C3SC = 0x24;                // Center-aligned low-true pulses
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM2C3VH = (byte) (val>>8);     // set duty cycle for Channel 3
           FTM2C3VL = (byte) (val);
      break;
        
      case 4:
           FTM2C4SC = 0x24;                // Center-aligned low-true pulses
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM2C4VH = (byte) (val>>8);     // set duty cycle for Channel 4
           FTM2C4VL = (byte) (val);
      break;
        
      case 5:
           FTM2C5SC = 0x24;                // Center-aligned low-true pulses
           if (MOD>100){
              val = DUTY_CYCLE*(MOD/100);
           }
           else {
              val = DUTY_CYCLE*MOD/100;
           }
           
           FTM2C5VH = (byte) (val>>8);     // set duty cycle for Channel 5
           FTM2C5VL = (byte) (val);
      break;
        
      default:
      break;      
    }
    
    FTM2SC   = (byte) (0x28|(CLK_PS&0x07));  // All center-aligned, bus frequency/Prescale, start timer
} //end FTM2Cx_PWM

/************************
   Main Program Loop
************************/
void main(void) {

  byte tempA;
  
  some_key_pressed = no;
  key_press_debounced = no;
  
  InitSystems();
  EnablePullups();
  InitADC();
  InitKBI(); 

  SRTISC  = 0x02;                          // 16ms period, no interrupt

  SetADC(0x08,0x01);                       // ADC8, enable interrupt

  EnableInterrupts;
  

  for (;;) {
      if (some_key_pressed&&key_press_debounced) {
         
                                          
         tempA = PTGD&0x0F;                // get the key pressed
         
         if ((tempA&0x01)==0x00) {         // SW1 key pressed

            FTM1Cx_PWM(2,25,1000,0x07);    // LED1 25% duty cycle ON.
            while ((PTGD&0x01)==0x00) {
                  ;                        // Wait till key is released
            }
            FTM1Cx_PWM(2,0,1000,0x07);     // LED1 OFF
         }                                
         if ((tempA&0x02)==0x00) {         // SW2 key pressed

            FTM1Cx_PWM(3,50,1000,0x07);    // LED2 50% duty cycle ON.
            while ((PTGD&0x02)==0x00) {
                  ;                        // Wait till key is released
            }
            FTM1Cx_PWM(3,0,1000,0x07);     // LED2 OFF
         }                                
         if ((tempA&0x04)==0x00) {         // SW3 key pressed

            FTM2Cx_PWM(0,75,1000,0x07);    // LED3 75% duty cycle ON.
            while ((PTGD&0x04)==0x00) {
                  ;                        // Wait till key is released
            }
            FTM2Cx_PWM(0,0,1000,0x07);     // LED3 OFF
         }                                
         if ((tempA&0x08)==0x00) {         // SW4 key pressed

            FTM2Cx_PWM(1,90,1000,0x07);    // LED4 90% duty cycle ON.
            while ((PTGD&0x08)==0x00) {
                  ;                        // Wait till key is released
            }
            FTM2Cx_PWM(1,0,1000,0x07);     // LED4 OFF
         }                                
         
         key_press_debounced = no;

         SRTISC_RTIACK   = 1;                     // Clear RTIF flag
         SRTISC  = 0x11;                          // Enable RTIE
         
         while (!key_press_debounced) {
               ;                           // debounce key
         }

         some_key_pressed = no;
         key_press_debounced = no;

         KBISC_KBIE = 1;                   // enable KBI interrupts
      }                     
  }
} //end main

/*********************************************************************
**  Interrupt Service Routines
*********************************************************************/
interrupt VectorNumber_Vadc1 void   ADC_ISR(void) {

  word adc_val;
    
  SRTISC_RTIACK = 1;                       // ACK RTI int
  adc_val  = (word) (ADC1RH<<8);           // Read ADC value, clear flag
  adc_val |= (word) (ADC1RL);              // 8-bit, ignore high byte
  
  FTM1Cx_PWM(4,(byte)(((0xFFFF-adc_val)&0x00FF)>>1),666,0x06); 
  FTM1Cx_PWM(5,(byte)((adc_val&0x00FF)>>1),666,0x06);
}

interrupt VectorNumber_Vkeyboard void   KBIx_ISR(void) {

  some_key_pressed  = yes;
  key_press_debounced = no;
  
  KBISC_KBIE  = 0;                         // Disable KBI interrupts
  
  KBISC_KBACK = 1;                         // Clear KBACK

  SRTISC_RTIACK   = 1;                     // Clear RTIF flag
  SRTISC  = 0x11;                          // Enable RTIE
}

interrupt VectorNumber_Vrti void   RTI_ISR(void) {
  if (some_key_pressed) {
     SRTISC  = 0x01;                       // Disable RTIE
     key_press_debounced = yes;
  }
  SRTISC_RTIACK = 1;                       // clear flag
}
